/*-
 * Copyright (c) 2008 FUKAUMI Naoki.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/mman.h>
#include <err.h>
#include <fcntl.h>
#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

int
main(int argc, char *argv[])
{
    char* buffer = NULL;
    size_t syssize;

	union {
		uint32_t w[47];
		uint8_t b[188];
	} buf;
	uint32_t *xor, *np;
	uint8_t *header;
	size_t offset;
	int i, null = 0, xorfd;

	if (argc > 2) {
		fprintf(stderr, "usage: %s [file] < in.ts > out.ts\n", argv[0]);
		exit(EXIT_FAILURE);
	}

	if (argc == 2) {
		if ((xorfd = open(argv[1], O_RDONLY)) == -1)
			err(EXIT_FAILURE, argv[1]);
	} else {
		if ((xorfd = open("SKNET_HDTV_BDA.sys", O_RDONLY)) == -1)
			err(EXIT_FAILURE, "SKNET_HDTV_BDA.sys");
	}

        syssize = 0x12da8 + 184;
        buffer = (char*)malloc( syssize );
        if( read( xorfd, buffer, syssize ) != syssize ){
            fprintf( stderr, "could not read SKNET_HDTV_BDA.sys\n");
            goto err;
        }

        header = buffer;
	if (header[0] == 0x47 && header[1] == 0x1f && header[2] == 0xff) {
		null = 1; /* XOR'ed null packet */
		offset = 4;
	} else if (header[0x2c8] == 0x8d && header[0x2c9] == 0x5f)
		offset = 0xf628; /* CD 1.0 */
	else if (header[0x2c0] == 0x31 && header[0x2c1] == 0xe6)
		offset = 0x12da8; /* CD 1.1 */
	else {
		fprintf(stderr, "unknown file\n");
		exit(EXIT_FAILURE);
	}

        fprintf( stderr, "offset = 0x%x\n", offset );

	if (null == 1) {
		if ((xor = malloc(184)) == NULL)
			err(EXIT_FAILURE, "malloc()");
                np = (uint32_t *)( buffer + offset );

		for (i = 0; i < 46; i++) {
			xor[i] = np[i] ^ 0xffffffff;
		}

		close(xorfd);
	} else {
            xor = (uint32_t *)( buffer + offset );
	}

	memset(buf.b, 0, 188);

 sync:
	while (read(STDIN_FILENO, buf.b, 1) == 1) {
		if (buf.b[0] == 0x47) /* XXX: may not be sync byte */
			if (read(STDIN_FILENO, &buf.b[1], 187) == 187)
				break;
		fprintf(stderr, ".");
	}

	if (buf.b[0] != 0x47)
		goto err;

	for (;;) {
		for (i = 1; i < 47; i++)
			buf.w[i] ^= xor[i - 1];
		write(STDOUT_FILENO, buf.w, 188);
		if (read(STDIN_FILENO, buf.w, 188) != 188)
			break;
		if (buf.b[0] != 0x47)
			goto sync;
	}

 err:
	if (null == 1) {
		free(xor);
	} else {
		close(xorfd);
	}

        if( buffer ) free( buffer );
	return EXIT_SUCCESS;
}
